"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.readPrismaDeclarations = void 0;
const tslib_1 = require("tslib");
const promises_1 = tslib_1.__importDefault(require("fs/promises"));
const namespace_1 = require("../helpers/namespace");
const source_path_1 = require("./source-path");
async function readPrismaDeclarations(nsName, clientOutput, overrideTarget, schemaTarget) {
    const declaration = {};
    declaration.sourcePath = (0, source_path_1.getSourcePath)(clientOutput, overrideTarget, schemaTarget);
    // reads the file content
    declaration.content = await promises_1.default.readFile(declaration.sourcePath, 'utf-8');
    // creates a empty list of changes
    declaration.changeset = [];
    //  a 1   1 a
    //  s 2   2 s
    //  d 3   3 s <- (start: 1, end: 3, text: `s`) changed `s` to `ss` (start: 1, wide: 2)
    //    4   4 d
    //    5   5
    //  a 6   6
    //  s 7   7 a
    //  d 8   8 s
    //    9   9 d
    declaration.replacer = (start, end, text) => {
        // Adds a trailing space
        if (text[0] !== ' ') {
            text = ' ' + text;
        }
        for (const change of declaration.changeset) {
            if (start > change.start) {
                start += change.diff;
                end += change.diff;
            }
        }
        // replaces the file content at the correct position
        declaration.content =
            declaration.content.slice(0, start) + text + declaration.content.slice(end);
        // adds the change to the list
        declaration.changeset.push({
            start,
            diff: start - end + text.length
        });
    };
    declaration.update = async () => {
        await promises_1.default.writeFile(declaration.sourcePath, (0, namespace_1.createNamespace)(nsName) + declaration.content);
    };
    return declaration;
}
exports.readPrismaDeclarations = readPrismaDeclarations;
//# sourceMappingURL=reader.js.map