"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.handleModule = void 0;
const tslib_1 = require("tslib");
const typescript_1 = tslib_1.__importDefault(require("typescript"));
const handle_signature_1 = require("../helpers/handle-signature");
const regex_1 = require("../helpers/regex");
async function handleModule(module, replacer, models, nsName) {
    const namespace = module
        .getChildren()
        .find((n) => n.kind === typescript_1.default.SyntaxKind.ModuleBlock);
    if (!namespace) {
        throw new Error('Prisma namespace could not be found');
    }
    for (const statement of namespace.statements) {
        const typeAlias = statement;
        // Filters any statement that isn't a export type declaration
        if (statement.kind !== typescript_1.default.SyntaxKind.TypeAliasDeclaration ||
            typeAlias.type.kind !== typescript_1.default.SyntaxKind.TypeLiteral) {
            continue;
        }
        const typeAliasName = typeAlias.name.getText();
        const typeAliasType = typeAlias.type;
        // May includes the model name but is not the actual model, like
        // UserCreateWithoutPostsInput for Post model. that's why we need
        // to check if the model name is in the regex
        const model = models.find((m) => m.regexps.some((r) => r.test(typeAliasName)));
        if (!model) {
            continue;
        }
        const fields = model.fields.filter((f) => f.documentation?.match(regex_1.JSON_REGEX));
        for (const member of typeAliasType.members) {
            if (member.kind !== typescript_1.default.SyntaxKind.PropertySignature) {
                continue;
            }
            const signature = member;
            const fieldName = member.name?.getText();
            const field = fields.find((f) => f.name === fieldName);
            if (!field || !fieldName) {
                continue;
            }
            if (!signature.type) {
                throw new Error(`No type found for field ${fieldName} at model ${typeAliasName}`);
            }
            const typename = field.documentation?.match(regex_1.JSON_REGEX)?.[1];
            if (!typename) {
                throw new Error(`No typename found for field ${fieldName} at model ${typeAliasName}`);
            }
            (0, handle_signature_1.replaceSignature)(signature.type, typename, nsName, replacer, fieldName, model.name, typeAliasName);
        }
    }
}
exports.handleModule = handleModule;
//# sourceMappingURL=module.js.map