"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.handleTypeAlias = void 0;
const tslib_1 = require("tslib");
const typescript_1 = tslib_1.__importDefault(require("typescript"));
const replace_object_1 = require("../helpers/replace-object");
const model_payload_1 = require("./model-payload");
async function handleTypeAlias(typeAlias, replacer, models, nsName) {
    // enum declarations
    if (typeAlias.getChildAt(4)?.getText().startsWith('(typeof')) {
        return;
    }
    const name = typeAlias.name.getText();
    const model = models.find((m) => m.name === name);
    // not a model with typed json fields
    if (!model) {
        const modelPayload = models.find((m) => `${m.name}Payload` === name);
        if (modelPayload) {
            return (0, model_payload_1.handleModelPayload)(typeAlias, replacer, modelPayload, nsName);
        }
        return;
    }
    const object = typeAlias.type;
    // not a type literal. TODO: there is a way of this happen?
    if (object.kind !== typescript_1.default.SyntaxKind.TypeLiteral) {
        // For `model`Payload types. They were handled in `handleModelPayload` call
        if (object.kind === typescript_1.default.SyntaxKind.IndexedAccessType) {
            return;
        }
        throw new Error(`Provided object is not a type literal: ${object.getText()}`);
    }
    (0, replace_object_1.replaceObject)(model, object, nsName, replacer, typeAlias.name.getText());
}
exports.handleTypeAlias = handleTypeAlias;
//# sourceMappingURL=type-alias.js.map