"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.handleModelPayload = void 0;
const tslib_1 = require("tslib");
const typescript_1 = tslib_1.__importDefault(require("typescript"));
const replace_object_1 = require("../helpers/replace-object");
async function handleModelPayload(typeAlias, replacer, model, nsName) {
    const type = typeAlias.type;
    if (type.kind !== typescript_1.default.SyntaxKind.TypeLiteral) {
        throw new Error(`Provided object is not a type literal: ${type.getText()}`);
    }
    const scalarsField = type.members.find((m) => m.name?.getText() === 'scalars');
    // Besides `scalars` field, the `objects` field also exists, but we don't need to handle it
    // because it just contains references to other <model>Payloads that we already change separately
    if (!scalarsField) {
        return;
    }
    const object = scalarsField?.type
        ?.typeArguments?.[0];
    if (!object) {
        throw new Error(`Payload scalars could not be resolved: ${type.getText()}`);
    }
    (0, replace_object_1.replaceObject)(model, object, nsName, replacer, typeAlias.name.getText());
}
exports.handleModelPayload = handleModelPayload;
//# sourceMappingURL=model-payload.js.map